#!/usr/bin/env python
#ident "@(#) $Id: setup.py,v 1.19 2002/10/27 03:47:27 ghaering Exp $"
#-----------------------------------------------------------------------+
# Name:		setup.py						|
#									|
# Synopsis:	python setup.py build	# Build the module.		|
#		python setup.py install	# Install the module.		|
#									|
#		See http://www.python.org/sigs/distutils-sig/doc/ for	|
#		more information on using distutils to install Python	|
#		programs and modules.					|
#									|
# Description:	Setup script (using the distutils framework) for	|
#		pyPgSQL.						|
#=======================================================================|
# Copyright 2001, 2002 by Gerhard Haering.				|
# All rights reserved.							|
#									|
# Permission to use, copy, modify, and distribute this software and its	|
# documentation for any purpose and without fee is hereby granted, pro-	|
# vided that the above copyright notice appear in all copies and that	|
# both that copyright notice and this permission notice appear in sup-	|
# porting documentation, and that the copyright owner's name not be	|
# used in advertising or publicity pertaining to distribution of the	|
# software without specific, written prior permission.			|
#									|
# THE AUTHOR(S) DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,	|
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND  FITNESS.  IN	|
# NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY SPECIAL, INDIRECT OR	|
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS	|
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE	|
# OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE	|
# USE OR PERFORMANCE OF THIS SOFTWARE.					|
#=======================================================================|
# People who have worked on this code.					|
#									|
# Ini Name								|
# --- ----------------------------------------------------------------- |
#  gh Gerhard Haering <gerhard.haering@gmx.de>				|
# bga Billy G. Allie <bill.allie@mug.org>				|
#=======================================================================|
# Revision History:							|
#									|
# Date      Ini Description						|
# --------- --- ------------------------------------------------------- |
# 06OCT2002 bga Added support for UnixWare7 and OpenUNIX 8.		|
# 16SEP2002 gh  Total rework to make setup.py work out of the box for   |
#               most popular platforms, while still allowing flexibi-   |
#               lity through setup.config.                              |
# 29AUG2001 gh  Reflected changed PostgreSQL win32 build process (Post- |
#	        greSQL now built with mingw32)			        |
# 28AUG2001 bga Add include_dirs and lib_dirs for building on cygwin.	|
#		This change should allow pyPgSQL to build 'out of the	|
#		box' on MS Windows using the cygwin environment.	|
# 20AUG2001 bga Modified to include the new source files.		|
#	    --- Added code to determine any needed runtime library	|
#		search directories.					|
# 04AUG2001 gh  Modified to include the files pgversion.c,		|
#		pymemstrdup.c and strtok.c.				|
# 26JUL2001 bga Modified to use included strtoll.c and strtoull.c files.|
#	    --- Added this flower box and cleaned up file.		|
# 22JUL2001 gh  Modified to build win32 version.			|
# 01JUN2001 gh	Initial version created by Gerhard Haering.		|
#-----------------------------------------------------------------------+
import os, os.path, sys

from distutils.core import setup
from distutils.extension import Extension

__version__ = "2.3b1"

# Define the runtime library path for this module.  It starts out as None.

class UnknownPlatformError(Exception):
    pass

def getLinuxDistribution():
    f = open("/etc/issue")
    issue = f.read()
    f.close()
    if issue.find("Debian GNU") >= 0:
	return "Debian"
    elif issue.find("Red Hat Linux") >= 0:
	return "Redhat"
    elif issue.find("SuSE Linux") >= 0:
	return "SuSE"
    elif issue.find("Mandrake") >= 0:
	return "Mandrake"
    else:
	return "unknown"

class Config:
    def __init__(self):
	# Default settings, may be overriden for specific platforms
	self.pypgsql_rt_dirs = None
	self.optional_libs = ["pq"]
	self.modname = "pyPgSQL.libpq.libpqmodule"

	self.sources = ["libpqmodule.c",  "pgboolean.c",
			"pgint2object.c", "pgint8object.c",
			"pgversion.c",    "pglargeobject.c",
			"pgnotify.c",     "pgconnection.c",
			"pgresult.c",     "pymemstrdup.c",
			"port/strtoll.c", "port/strtoull.c",
			"port/strtok.c"]

	imported_locals = {}
	execfile("setup.config", {}, imported_locals)
	self.use_custom = len(imported_locals) > 0
	if self.use_custom:
	    for k, v in imported_locals.items():
		self.__dict__[k] = v

def main():
    cfg = Config()

    if cfg.use_custom:
	pass
    elif sys.platform == "linux2":
	distribution = getLinuxDistribution()
	if distribution == "Redhat":
	    cfg.include_dirs = ["/usr/include"]
	    cfg.library_dirs = ["/usr/lib"]
	elif distribution == "Debian":
	    cfg.include_dirs = ["/usr/include/postgresql"]
	    cfg.library_dirs = ["/usr/lib"]
	elif distribution in ("SuSE", "Mandrake"):
	    cfg.include_dirs = ["/usr/include/pgsql"]
	    cfg.library_dirs = ["/usr/lib"]
	else:
	    raise UnknownPlatformError, "Unknown Linux distribution."
	# XXX: This is an ugly hack to make bdist_rpm find the include files.
	cfg.include_dirs.append("../" * 5)
    elif sys.platform[:8] == "unixware":
	LOCALBASE = os.environ.get('LOCALBASE', '/usr/local/pgsql')
	cfg.include_dirs = ['%s/include' % LOCALBASE]
	cfg.library_dirs = ['%s/lib' % LOCALBASE]
	cfg.pypgsql_rt_dirs = cfg.library_dirs
    elif sys.platform == "freebsd4":
	LOCALBASE = os.environ.get('LOCALBASE', '/usr/local')
	cfg.include_dirs = ['%s/include' % LOCALBASE]
	cfg.library_dirs = ['%s/lib' % LOCALBASE]
    elif sys.platform == "openbsd3":
	LOCALBASE = os.environ.get('LOCALBASE', '/usr/local')
	cfg.include_dirs = ['%s/include/postgresql' % LOCALBASE]
	cfg.library_dirs = ['%s/lib' % LOCALBASE]
    elif sys.platform == "netbsd1":
	LOCALBASE = os.environ.get('LOCALBASE', '/usr/pkg')
	cfg.include_dirs = ['%s/include/pgsql' % LOCALBASE]
	cfg.library_dirs = ['%s/lib' % LOCALBASE]
    elif sys.platform == "cygwin":
	cfg.include_dirs  = ["/usr/include/postgresql"]
	cfg.library_dirs  = ["/usr/lib"]
    elif sys.platform == "darwin": # Mac OS X
	cfg.include_dirs = ["/usr/local/pgsql/include"]
	cfg.library_dirs = ["/usr/local/pgsql/lib"]
	cfg.optional_libs += ["ssl", "crypto"]
    elif sys.platform == "win32":
	# This works with the PostgreSQL source tree, so it's a bit ugly ...
	win_pg_build_root = os.getenv("PG_SRC", "../postgresql")

	cfg.include_dirs = [os.path.join(win_pg_build_root, p) for p in
				["src/include",
				 "src/include/libpq",
				 "src",
				 "src/interfaces/libpq"]]

	cfg.library_dirs = [win_pg_build_root + "/src/interfaces/libpq/"]
	cfg.optional_libs += ["wsock32", "advapi32"]
	cfg.modname="pyPgSQL.libpq.libpq"
    else:
	raise UnknownPlatformError, "Unknown platform."

    setup (
	name = "pyPgSQL",
	version = __version__,
	description = \
	    "pyPgSQL - A Python DB-API 2.0 compliant interface to PostgreSQL.",
	author = "Billy G. Allie",
	author_email = "ballie01@sourceforge.net",
	url = "http://pypgsql.sourceforge.net/",
	licence = "Python",
	packages = ["pyPgSQL", "pyPgSQL.libpq"],
	ext_modules = [Extension(
	    name=cfg.modname,
	    sources = cfg.sources,
	    include_dirs = cfg.include_dirs,
	    library_dirs = cfg.library_dirs,
	    runtime_library_dirs = cfg.pypgsql_rt_dirs,
	    libraries = cfg.optional_libs
	    )]
    )

if __name__ == "__main__":
    try:
	main()
    except UnknownPlatformError, reason:
	print "Cannot build pyPgSQL:", reason
	print
	print "setup.py can't guess the correct settings for your platform."
	print
	print "Please manually edit setup.conf appropriately for your platform."
	print "If you have any problems, please ask on the pypgsql-users mailing list."
	print
	print "Once you've managed to build pyPgSQL on your platform, please submit a"
	print "patch to setup.py so that we can support your platform by default in"
	print "the future. The pyPgSQL developers are happy to assist you in doing so."
